import numpy as np
from sklearn import datasets
from sklearn import svm
from sklearn.dummy import DummyClassifier
from sklearn.metrics import accuracy_score
from sklearn.model_selection import train_test_split
from sklearn_helper.model_finder.BestFitFinder import BestFitFinder


if __name__ == "__main__":
    digits = datasets.load_digits()
    X_train, X_test, y_train, y_test = train_test_split(digits.data, digits.target, random_state=0, test_size=0.3)

    models = {
        "DummyClassifier": {
            "model": DummyClassifier()
        },
        "SVC": {
            "model": svm.SVC()
        },
        "Tuned SVC": {
            "model": svm.SVC(),
            "params": {
                "gamma": np.linspace(0, 0.1, num=10),
                "C": range(1, 10)
            }
        },
    }
    model_finder = BestFitFinder(models, main_metric="accuracy")
    model = model_finder.evaluate(X_train, y_train)

    print(model)
    print(accuracy_score(model.predict(X_test), y_test))
